@@ -4,44 +4,6 @@ import errno
 from urlparse import urlparse
 from program import *
 
-def popen(command, stdin=None, **kwargs):
-    # print for debugging
-    info('Exec "'+' '.join(command)+'" in '+os.getcwd())
-    try:
-        proc = subprocess.Popen(command, **kwargs)
-    except OSError as e:
-        if e[0] == errno.ENOENT:
-            error(
-                "Could not execute \"%s\".\n"
-                "Please verify that it's installed and accessible from your current path by executing \"%s\".\n" % (command[0], command[0]), e[0])
-        else:
-            raise e
-
-    if proc.wait() != 0:
-        raise ProcessException(proc.returncode, command[0], ' '.join(list(command)), os.getcwd())
-
-def pquery(command, stdin=None, **kwargs):
-    if very_verbose:
-        info('Query "'+' '.join(command)+'" in '+os.getcwd())
-    try:
-        proc = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, **kwargs)
-    except OSError as e:
-        if e[0] == errno.ENOENT:
-            error(
-                "Could not execute \"%s\".\n"
-                "Please verify that it's installed and accessible from your current path by executing \"%s\".\n" % (command[0], command[0]), e[0])
-        else:
-            raise e
-
-    stdout, _ = proc.communicate(stdin)
-
-    if very_verbose:
-        log(str(stdout).strip()+"\n")
-
-    if proc.returncode != 0:
-        raise ProcessException(proc.returncode, command[0], ' '.join(command), os.getcwd())
-
-    return stdout
 
 def formaturl(url, format="default"):
     url = "%s" % url
@@ -57,9 +19,12 @@ def formaturl(url, format="default"):
             if format == "ssh":
                 url = 'ssh://%s%s/%s.git' % (m.group(2) or 'git@', m.group(6), m.group(7))
             elif format == "http":
-                url = 'http://%s%s/%s' % (m.group(2) if (m.group(2) and (m.group(5) or m.group(3) != 'git')) else '', m.group(6), m.group(7))
+                url = 'http://%s%s/%s' % (
+                m.group(2) if (m.group(2) and (m.group(5) or m.group(3) != 'git')) else '', m.group(6), m.group(7))
             elif format == "https":
-                url = 'https://%s%s/%s%s' % (m.group(2) if (m.group(2) and (m.group(5) or m.group(3) != 'git')) else '', m.group(6), m.group(7), m.group(8))
+                url = 'https://%s%s/%s%s' % (
+                m.group(2) if (m.group(2) and (m.group(5) or m.group(3) != 'git')) else '', m.group(6), m.group(7),
+                m.group(8))
         else:
             m = re.match(regex_hg_url, url)
             if m:
@@ -71,14 +36,19 @@ def formaturl(url, format="default"):
                     url = 'https://%s/%s' % (m.group(2), m.group(3))
     return url
 
+
 # Handling for multiple version controls
 scms = {}
+
+
 def scm(name):
     def _scm(cls):
         scms[name] = cls()
         return cls
+
     return _scm
 
+
 # pylint: disable=no-self-argument, no-method-argument, no-member, no-self-use, unused-argument
 @scm('git')
 @staticclass
@@ -102,24 +72,26 @@ class Git(object):
             rmtree_readonly(os.path.join('.git', 'logs'))
 
     def clone(url, name=None, depth=None, protocol=None):
-        popen([git_cmd, 'clone', formaturl(url, protocol), name] + (['--depth', depth] if depth else []) + (['-v'] if very_verbose else ([] if verbose else ['-q'])))
+        popen([git_cmd, 'clone', formaturl(url, protocol), name] + (['--depth', depth] if depth else []) + (
+        ['-v'] if very_verbose else ([] if verbose else ['-q'])))
 
     def add(dest):
-        info("Adding reference "+dest)
+        info("Adding reference " + dest)
         try:
             popen([git_cmd, 'add', dest] + (['-v'] if very_verbose else []))
         except ProcessException:
             pass
 
     def remove(dest):
-        info("Removing reference "+dest)
+        info("Removing reference " + dest)
         try:
             popen([git_cmd, 'rm', '-f', dest] + ([] if very_verbose else ['-q']))
         except ProcessException:
             pass
 
     def commit(msg=None):
-        popen([git_cmd, 'commit', '-a'] + (['-m', msg] if msg else []) + (['-v'] if very_verbose else ([] if verbose else ['-q'])))
+        popen([git_cmd, 'commit', '-a'] + (['-m', msg] if msg else []) + (
+        ['-v'] if very_verbose else ([] if verbose else ['-q'])))
 
     def publish(all_refs=None):
         if all_refs:
@@ -130,11 +102,12 @@ class Git(object):
             if remote and branch:
                 popen([git_cmd, 'push', remote, branch] + (['-v'] if very_verbose else ([] if verbose else ['-q'])))
             else:
-                err = "Unable to publish outgoing changes for \"%s\" in \"%s\".\n" % (os.path.basename(os.getcwd()), os.getcwd())
+                err = "Unable to publish outgoing changes for \"%s\" in \"%s\".\n" % (
+                os.path.basename(os.getcwd()), os.getcwd())
                 if not remote:
-                    error(err+"The local repository is not associated with a remote one.", 1)
+                    error(err + "The local repository is not associated with a remote one.", 1)
                 if not branch:
-                    error(err+"Working set is not on a branch.", 1)
+                    error(err + "Working set is not on a branch.", 1)
 
     def fetch():
         info("Fetching revisions from remote repository to \"%s\"" % os.path.basename(os.getcwd()))
@@ -142,9 +115,10 @@ class Git(object):
 
     def discard(clean_files=False):
         info("Discarding local changes in \"%s\"" % os.path.basename(os.getcwd()))
-        pquery([git_cmd, 'reset', 'HEAD'] + ([] if very_verbose else ['-q'])) # unmarks files for commit
-        pquery([git_cmd, 'checkout', '.'] + ([] if very_verbose else ['-q'])) # undo  modified files
-        pquery([git_cmd, 'clean', '-fd'] + (['-x'] if clean_files else []) + (['-q'] if very_verbose else ['-q'])) # cleans up untracked files and folders
+        pquery([git_cmd, 'reset', 'HEAD'] + ([] if very_verbose else ['-q']))  # unmarks files for commit
+        pquery([git_cmd, 'checkout', '.'] + ([] if very_verbose else ['-q']))  # undo  modified files
+        pquery([git_cmd, 'clean', '-fd'] + (['-x'] if clean_files else []) + (
+        ['-q'] if very_verbose else ['-q']))  # cleans up untracked files and folders
 
     def merge(dest):
         info("Merging \"%s\" with \"%s\"" % (os.path.basename(os.getcwd()), dest))
@@ -156,13 +130,14 @@ class Git(object):
         info("Checkout \"%s\" in %s" % (rev, os.path.basename(os.getcwd())))
         branch = None
         refs = Git.getrefs(rev)
-        for ref in refs: # re-associate with a local or remote branch (rev is the same)
+        for ref in refs:  # re-associate with a local or remote branch (rev is the same)
             m = re.match(r'^(.*?)\/(.*?)$', ref)
-            if m and m.group(2) != "HEAD": # matches origin/<branch> and isn't HEAD ref
-                if not os.path.exists(os.path.join('.git', 'refs', 'heads', m.group(2))): # okay only if local branch with that name doesn't exist (git will checkout the origin/<branch> in that case)
+            if m and m.group(2) != "HEAD":  # matches origin/<branch> and isn't HEAD ref
+                if not os.path.exists(os.path.join('.git', 'refs', 'heads', m.group(
+                        2))):  # okay only if local branch with that name doesn't exist (git will checkout the origin/<branch> in that case)
                     branch = m.group(2)
             elif ref != "HEAD":
-                branch = ref # matches local branch and isn't HEAD ref
+                branch = ref  # matches local branch and isn't HEAD ref
 
             if branch:
                 info("Revision \"%s\" matches a branch \"%s\" reference. Re-associating with branch" % (rev, branch))
@@ -190,9 +165,9 @@ class Git(object):
             else:
                 err = "Unable to update \"%s\" in \"%s\"." % (os.path.basename(os.getcwd()), os.getcwd())
                 if not remote:
-                    info(err+" The local repository is not associated with a remote one.")
+                    info(err + " The local repository is not associated with a remote one.")
                 if not branch:
-                    info(err+" Working set is not on a branch.")
+                    info(err + " Working set is not on a branch.")
 
     def status():
         return pquery([git_cmd, 'status', '-s'] + (['-v'] if very_verbose else []))
@@ -257,7 +232,7 @@ class Git(object):
         remotes = Git.getremotes()
         for remote in remotes:
             url = remote[1]
-            if remote[0] == "origin": # Prefer origin URL
+            if remote[0] == "origin":  # Prefer origin URL
                 break
         return formaturl(url)
 
@@ -279,7 +254,7 @@ class Git(object):
         for line in lines:
             m = re.match(r'^(.+)\s+(.+)$', line)
             if m and (not rev or m.group(1).startswith(rev)):
-                if re.match(r'refs\/(heads|remotes)\/', m.group(2)): # exclude tags
+                if re.match(r'refs\/(heads|remotes)\/', m.group(2)):  # exclude tags
                     result.append(m.group(1) if ret_rev else re.sub(r'refs\/(heads|remotes)\/', '', m.group(2)))
         return result
 
@@ -301,7 +276,7 @@ class Git(object):
                 os.mkdir(ignore_file_parent_directory)
 
             with open(Git.ignore_file, 'w') as f:
-                f.write('\n'.join(ignores)+'\n')
+                f.write('\n'.join(ignores) + '\n')
         except IOError:
             error("Unable to write ignore file in \"%s\"" % os.path.join(os.getcwd(), Git.ignore_file), 1)
 
@@ -322,6 +297,7 @@ class Git(object):
                     f.write(dest.replace("\\", "/") + '\n')
             except IOError:
                 error("Unable to write ignore file in \"%s\"" % os.path.join(os.getcwd(), Git.ignore_file), 1)
+
     def unignore(dest):
         try:
             with open(Git.ignore_file) as f:
@@ -341,6 +317,7 @@ class Git(object):
             except IOError:
                 error("Unable to write ignore file in \"%s\"" % os.path.join(os.getcwd(), Git.ignore_file), 1)
 
+
 # Repository object
 class Repo(object):
     is_local = False
@@ -369,7 +346,7 @@ class Repo(object):
         elif m_bld_url:
             repo.name = os.path.basename(path or m_bld_url.group(7))
             repo.path = os.path.abspath(path or os.path.join(os.getcwd(), repo.name))
-            repo.url = m_bld_url.group(1)+'/builds'
+            repo.url = m_bld_url.group(1) + '/builds'
             repo.rev = m_bld_url.group(8)
             repo.is_build = True
         elif m_repo_url:
@@ -403,7 +380,8 @@ class Repo(object):
         m_bld_url = re.match(regex_build_url, ref.strip().replace('\\', '/'))
         if not (m_local or m_bld_url or m_repo_url):
             warning(
-                "File \"%s\" in \"%s\" uses a non-standard .comp or .codes file extension, which is not compatible with the aos build tools.\n" % (os.path.basename(lib), os.path.split(lib)[0]))
+                "File \"%s\" in \"%s\" uses a non-standard .comp or .codes file extension, which is not compatible with the aos build tools.\n" % (
+                os.path.basename(lib), os.path.split(lib)[0]))
             return False
         else:
             return cls.fromurl(ref, lib[:lib.rfind('.')])
@@ -418,7 +396,8 @@ class Repo(object):
         m_bld_url = re.match(regex_build_url, ref.strip().replace('\\', '/'))
         if not (m_local or m_bld_url or m_repo_url):
             warning(
-                "File \"%s\" in \"%s\" uses a non-standard .lib file extension, which is not compatible with the aos build tools.\n" % (os.path.basename(code), os.path.split(code)[0]))
+                "File \"%s\" in \"%s\" uses a non-standard .lib file extension, which is not compatible with the aos build tools.\n" % (
+                os.path.basename(code), os.path.split(code)[0]))
             return False
         else:
             return cls.fromurl(ref, code[:code.rfind('.')])
@@ -435,12 +414,12 @@ class Repo(object):
 
         repo.path = os.path.abspath(path)
         repo.name = os.path.basename(repo.path)
-        
+
         cache_cfg = Global().get_cfg('CACHE', '')
         if cache_repositories and cache_cfg and cache_cfg != 'none' and cache_cfg != 'off' and cache_cfg != 'disabled':
             loc = cache_cfg if (cache_cfg and cache_cfg != 'on' and cache_cfg != 'enabled') else None
             repo.cache = loc or os.path.join(tempfile.gettempdir(), 'aos-repo-cache')
-        
+
         repo.sync()
 
         if repo.scm is None:
@@ -453,7 +432,7 @@ class Repo(object):
     @classmethod
     def isrepo(cls, path=None):
         for name, _ in scms.items():
-            if os.path.isdir(os.path.join(path, '.'+name)):
+            if os.path.isdir(os.path.join(path, '.' + name)):
                 return True
 
         return False
@@ -488,7 +467,7 @@ class Repo(object):
         #             break
         #     else:
         #         break
-        
+
         return pd.get_cfg(PATH_TYPE) if pd.get_cfg(PATH_TYPE) else "directory"
 
     @classmethod
@@ -496,9 +475,9 @@ class Repo(object):
         if rev is None or len(rev) == 0:
             return 'latest' + (' revision in the current branch' if ret_rev else '')
         elif re.match(r'^([a-fA-F0-9]{6,40})$', rev) or re.match(r'^([0-9]+)$', rev):
-            return 'rev' + (' #'+rev[0:12] if ret_rev else '')
+            return 'rev' + (' #' + rev[0:12] if ret_rev else '')
         else:
-            return 'branch' + (' '+rev if ret_rev else '')
+            return 'branch' + (' ' + rev if ret_rev else '')
 
     @classmethod
     def isurl(cls, url):
@@ -518,7 +497,7 @@ class Repo(object):
         if self.url:
             return (self.url.rstrip('/') + '/' +
                     (('' if self.is_build else '#') +
-                        self.rev if self.rev else ''))
+                     self.rev if self.rev else ''))
 
     def sync(self):
         self.url = None
@@ -557,7 +536,7 @@ class Repo(object):
 
     def getscm(self):
         for name, scm in scms.items():
-            if os.path.isdir(os.path.join(self.path, '.'+name)):
+            if os.path.isdir(os.path.join(self.path, '.' + name)):
                 return scm
 
     # Pass backend SCM commands and parameters if SCM exists
@@ -566,6 +545,7 @@ class Repo(object):
             if self.scm and hasattr(self.scm, method) and callable(getattr(self.scm, method)):
                 with cd(self.path):
                     return getattr(self.scm, method)(*args, **kwargs)
+
         return __scm_call
 
     def __getattr__(self, attr):
@@ -636,7 +616,7 @@ class Repo(object):
 
     def getlibs(self):
         for root, dirs, files in os.walk(self.path):
-            dirs[:] = [d for d in dirs  if not d.startswith('.')]
+            dirs[:] = [d for d in dirs if not d.startswith('.')]
             files[:] = [f for f in files if not f.startswith('.')]
 
             for f in files:
@@ -649,7 +629,7 @@ class Repo(object):
 
     def getcodes(self):
         for root, dirs, files in os.walk(self.path):
-            dirs[:] = [d for d in dirs  if not d.startswith('.')]
+            dirs[:] = [d for d in dirs if not d.startswith('.')]
             files[:] = [f for f in files if not f.startswith('.')]
 
             for f in files:
@@ -660,22 +640,25 @@ class Repo(object):
                     if f[:f.rfind('.')] in dirs:
                         dirs.remove(f[:f.rfind('.')])
 
-
     def write(self):
         if os.path.isfile(self.lib):
             with open(self.lib) as f:
                 lib_repo = Repo.fromurl(f.read().strip())
-                if (formaturl(lib_repo.url, 'https') == formaturl(self.url, 'https') # match URLs in common format (https)
-                        and (lib_repo.rev == self.rev                              # match revs, even if rev is None (valid for repos with no revisions)
-                             or (lib_repo.rev and self.rev
-                                 and lib_repo.rev == self.rev[0:len(lib_repo.rev)]))):  # match long and short rev formats
-                    #print self.name, 'unmodified'
+                if (formaturl(lib_repo.url, 'https') == formaturl(self.url,
+                                                                  'https')  # match URLs in common format (https)
+                        and (
+                                lib_repo.rev == self.rev  # match revs, even if rev is None (valid for repos with no revisions)
+                                or (lib_repo.rev and self.rev
+                                    and lib_repo.rev == self.rev[
+                                                        0:len(lib_repo.rev)]))):  # match long and short rev formats
+                    # print self.name, 'unmodified'
                     return
 
         ref = (formaturl(self.url, 'https').rstrip('/') + '/' +
-              (('' if self.is_build else '#') +
+               (('' if self.is_build else '#') +
                 self.rev if self.rev else ''))
-        action("Update reference \"%s\" -> \"%s\"" % (relpath(cwd_root, self.path) if cwd_root != self.path else self.name, ref))
+        action("Update reference \"%s\" -> \"%s\"" % (
+        relpath(cwd_root, self.path) if cwd_root != self.path else self.name, ref))
         with open(self.lib, 'wb') as f:
             f.write(ref + '\n')
 
@@ -683,17 +666,20 @@ class Repo(object):
         if os.path.isfile(self.code):
             with open(self.code) as f:
                 lib_repo = Repo.fromurl(f.read().strip())
-                if (formaturl(lib_repo.url, 'ssh') == formaturl(self.url, 'ssh') # match URLs in common format (ssh)
-                        and (lib_repo.rev == self.rev                              # match revs, even if rev is None (valid for repos with no revisions)
-                             or (lib_repo.rev and self.rev
-                                 and lib_repo.rev == self.rev[0:len(lib_repo.rev)]))):  # match long and short rev formats
-                    #print self.name, 'unmodified'
+                if (formaturl(lib_repo.url, 'ssh') == formaturl(self.url, 'ssh')  # match URLs in common format (ssh)
+                        and (
+                                lib_repo.rev == self.rev  # match revs, even if rev is None (valid for repos with no revisions)
+                                or (lib_repo.rev and self.rev
+                                    and lib_repo.rev == self.rev[
+                                                        0:len(lib_repo.rev)]))):  # match long and short rev formats
+                    # print self.name, 'unmodified'
                     return
 
         ref = (formaturl(self.url, 'ssh').rstrip('/') + '/' +
-              (('' if self.is_build else '#') +
+               (('' if self.is_build else '#') +
                 self.rev if self.rev else ''))
-        action("Update reference \"%s\" -> \"%s\"" % (relpath(cwd_root, self.path) if cwd_root != self.path else self.name, ref))
+        action("Update reference \"%s\" -> \"%s\"" % (
+        relpath(cwd_root, self.path) if cwd_root != self.path else self.name, ref))
         with open(self.code, 'wb') as f:
             f.write(ref + '\n')
 
@@ -706,7 +692,8 @@ class Repo(object):
 
     def get_cache(self, url):
         up = urlparse(formaturl(url, 'https'))
-        if self.cache and up and up.netloc and os.path.isdir(os.path.join(self.cache, up.netloc, re.sub(r'^/', '', up.path))):
+        if self.cache and up and up.netloc and os.path.isdir(
+                os.path.join(self.cache, up.netloc, re.sub(r'^/', '', up.path))):
             return os.path.join(self.cache, up.netloc, re.sub(r'^/', '', up.path))
 
     def set_cache(self, url):
@@ -717,7 +704,7 @@ class Repo(object):
                 if not os.path.isdir(cpath):
                     os.makedirs(cpath)
 
-                scm_dir = '.'+self.scm.name
+                scm_dir = '.' + self.scm.name
                 if os.path.isdir(os.path.join(cpath, scm_dir)):
                     rmtree_readonly(os.path.join(cpath, scm_dir))
                 shutil.copytree(os.path.join(self.path, scm_dir), os.path.join(cpath, scm_dir))
@@ -729,17 +716,20 @@ class Repo(object):
         err = None
         if (self.is_local or self.url is None) and not clean_deps:
             err = (
-                "Preserving local library \"%s\" in \"%s\".\nPlease publish this library to a remote URL to be able to restore it at any time."
-                "You can use --ignore switch to ignore all local libraries and update only the published ones.\n"
-                "You can also use --clean-deps switch to remove all local libraries. WARNING: This action cannot be undone." % (self.name, self.path))
+                    "Preserving local library \"%s\" in \"%s\".\nPlease publish this library to a remote URL to be able to restore it at any time."
+                    "You can use --ignore switch to ignore all local libraries and update only the published ones.\n"
+                    "You can also use --clean-deps switch to remove all local libraries. WARNING: This action cannot be undone." % (
+                    self.name, self.path))
         elif not clean and self.dirty():
             err = (
-                "Uncommitted changes in \"%s\" in \"%s\".\nPlease discard or stash them first and then retry update.\n"
-                "You can also use --clean switch to discard all uncommitted changes. WARNING: This action cannot be undone." % (self.name, self.path))
+                    "Uncommitted changes in \"%s\" in \"%s\".\nPlease discard or stash them first and then retry update.\n"
+                    "You can also use --clean switch to discard all uncommitted changes. WARNING: This action cannot be undone." % (
+                    self.name, self.path))
         elif not clean_deps and self.outgoing():
             err = (
-                "Unpublished changes in \"%s\" in \"%s\".\nPlease publish them first using the \"publish\" command.\n"
-                "You can also use --clean-deps to discard all local commits and replace the library with the one included in this revision. WARNING: This action cannot be undone." % (self.name, self.path))
+                    "Unpublished changes in \"%s\" in \"%s\".\nPlease publish them first using the \"publish\" command.\n"
+                    "You can also use --clean-deps to discard all local commits and replace the library with the one included in this revision. WARNING: This action cannot be undone." % (
+                    self.name, self.path))
 
         return (False, err) if err else (True, "OK")
 
@@ -747,14 +737,16 @@ class Repo(object):
         err = None
         if not os.path.isdir(self.path):
             err = (
-                "Library reference \"%s\" points to non-existing library in \"%s\"\n"
-                "You can use \"aos deploy\" to import the missing libraries.\n"
-                "You can also use \"aos sync\" to synchronize and remove all invalid library references." % (os.path.basename(self.lib), self.path))
+                    "Library reference \"%s\" points to non-existing library in \"%s\"\n"
+                    "You can use \"aos deploy\" to import the missing libraries.\n"
+                    "You can also use \"aos sync\" to synchronize and remove all invalid library references." % (
+                    os.path.basename(self.lib), self.path))
         elif not self.isrepo(self.path):
             err = (
-                "Library reference \"%s\" points to a folder \"%s\", which is not a valid repository.\n"
-                "You can remove the conflicting folder manually and use \"aos deploy\" to import the missing libraries\n"
-                "You can also remove library reference \"%s\" and use \"aos sync\" again." % (os.path.basename(self.lib), self.path, self.lib))
+                    "Library reference \"%s\" points to a folder \"%s\", which is not a valid repository.\n"
+                    "You can remove the conflicting folder manually and use \"aos deploy\" to import the missing libraries\n"
+                    "You can also remove library reference \"%s\" and use \"aos sync\" again." % (
+                    os.path.basename(self.lib), self.path, self.lib))
 
         if err:
             if show_warning:
